@@ -211,6 +211,7 @@ ipx_add_interface(int argc, char **argv)
 			strerror(errno));
 		break;
 	}
+	close(s);
 	exit(-1);
 }
 
@@ -236,6 +237,7 @@ ipx_delall_interface(int argc, char **argv)
 	fp = fopen("/proc/net/ipx_interface", "r");
 	if (fp == NULL)
 	{
+		close(s);
 		fprintf(stderr,
 			_("%s: Unable to open \"%s.\"\n"),
 			progname,
@@ -286,7 +288,8 @@ ipx_delall_interface(int argc, char **argv)
 			break;
 		}
 	}
-
+	fclose(fp);
+	close(s);
 	exit(0);
 }
 
@@ -320,8 +323,10 @@ ipx_del_interface(int argc, char **argv)
 	sipx->sipx_action = IPX_DLTITF;
 	sipx->sipx_family = AF_IPX;
 	result = ioctl(s, SIOCSIFADDR, &id);
-	if (result == 0)
+	if (result == 0) {
+		close(s);
 		exit(0);
+	}
 
 	switch (errno)
 	{
@@ -342,6 +347,7 @@ ipx_del_interface(int argc, char **argv)
 			strerror(errno));
 		break;
 	}
+	close(s);
 	exit(-1);
 }
 
@@ -375,6 +381,7 @@ ipx_check_interface(int argc, char **argv)
 	result = ioctl(s, SIOCGIFADDR, &id);
 	if (result == 0)
 	{
+		close(s);
 		printf(
 			      _("IPX Address for (%s, %s) is %08X:%02X%02X%02X%02X%02X%02X.\n"),
 			      argv[1], frame_types[fti].ft_name,
@@ -403,6 +410,7 @@ ipx_check_interface(int argc, char **argv)
 			strerror(errno));
 		break;
 	}
+	close(s);
 	exit(-1);
 }
 
