@@ -1,20 +1,28 @@
 /* ipxdump.c */
 
-/* Copyright 1996 Volker Lendecke, Goettingen, Germany
+/*  Copyright 1996 Volker Lendecke, Goettingen, Germany
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+
+    Revision History:
+
+	1.01	1999, December, 15	Petr Vandrovec <vandrove@vc.cvut.cz>
+			Use sigaction instead of signal, so ^C works
+			immediately and not after next frame comes
+			(bsd vs. sysv signal() semantic bug)
 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include "config.h"
@@ -74,6 +82,12 @@ static int filter = 0;
 static int allframes = 0;
 static IPXNode filter_node;
 
+static const char* progname;
+
+static void usage(void) {
+	fprintf(stderr, "usage: %s [-r] [-d device] [node]\n", progname);
+}
+
 static int exit_request = 0;
 static void
 int_handler()
@@ -92,13 +106,27 @@ main(int argc, char *argv[])
 	unsigned char buf[4096];
 	int length;
 	int opt;
+	struct sigaction saint;
 
-	signal(SIGINT, int_handler);
+	progname = strrchr(argv[0], '/');
+	if (progname)
+		progname++;
+	else
+		progname = argv[0];
 
-	while ((opt = getopt(argc, argv, "r")) != -1) {
+	saint.sa_handler = int_handler;
+	saint.sa_flags = 0;
+	sigemptyset(&saint.sa_mask);
+	sigaction(SIGINT, &saint, NULL);
+
+	while ((opt = getopt(argc, argv, "rd:h")) != -1) {
 		switch (opt) {
 			case 'r':allframes = 1; 
 				 break;
+			case 'd':device = optarg;
+				 break;
+			case 'h':usage();
+				 return 5;
 			case ':':;
 			case '?':break;
 			default: printf("?? unknown option returned by getopt\n");
@@ -109,16 +137,17 @@ main(int argc, char *argv[])
 	{
 		if (ipx_sscanf_node(argv[optind], filter_node) != 0)
 		{
-			fprintf(stderr, "usage: %s [-a] [node]\n", argv[0]);
+			usage();
 			exit(1);
 		}
 		filter = 1;
 	}
 	setvbuf(stdout, NULL, _IOLBF, 0);
-	if ((sd = socket(AF_INET, SOCK_PACKET, htons(ETH_P_ALL))) < 0)
+//	if ((sd = socket(AF_INET, SOCK_PACKET, htons(ETH_P_ALL))) < 0)
+	if ((sd = socket(AF_PACKET, SOCK_RAW, htons(ETH_P_ALL))) < 0)
 	{
 		perror("Can't get socket");
-		fprintf(stderr, "You must run %s as root\n", argv[0]);
+		fprintf(stderr, "You must run %s as root\n", progname);
 		exit(1);
 	}
 	/* SET PROMISC */
