@@ -19,14 +19,21 @@
 
 #include <unistd.h>
 #include <sys/types.h>
-#include <sys/socket.h>
+#include <ncp/ext/socket.h>
+/* TBD */
 #include <netinet/in.h>
+#ifdef HAVE_NETINET_IF_ETHER_H
 #include <netinet/if_ether.h>
-#include <strings.h>
+#else
+#include <linux/if_ether.h>
+#endif
+#include <string.h>
 #include <sys/ioctl.h>
-#include <net/if.h>
+#include <ncp/kernel/if.h>
 #include <signal.h>
+/* TBD */
 #include <netinet/ip.h>
+/* TBD */
 #include <netinet/tcp.h>
 #include <stdio.h>
 #include <time.h>
@@ -34,26 +41,22 @@
 #include <ctype.h>
 #include "ipxutil.h"
 
-#define __u8 u_int8_t
-#define __u16 u_int16_t 
-#define __u32 u_int32_t
-
 #define DUMPALLSAPS		/* #define if you want to dump all SAP's */
 
 struct ipx_address
 {
-	unsigned long net;
-	unsigned char node[IPX_NODE_LEN];
-	unsigned short sock;
+	IPXNet	net	__attribute__((packed));
+	IPXNode	node	__attribute__((packed));
+	IPXPort	sock	__attribute__((packed));
 };
 
 struct ipx_packet
 {
-	unsigned short ipx_checksum;
+	unsigned short ipx_checksum	__attribute__((packed));
 #define IPX_NO_CHECKSUM	0xFFFF
-	unsigned short ipx_pktsize;
-	unsigned char ipx_tctrl;
-	unsigned char ipx_type;
+	unsigned short ipx_pktsize	__attribute__((packed));
+	unsigned char ipx_tctrl		__attribute__((packed));
+	unsigned char ipx_type		__attribute__((packed));
 #define IPX_TYPE_UNKNOWN	0x00
 #define IPX_TYPE_RIP		0x01	/* may also be 0 */
 #define IPX_TYPE_SAP		0x04	/* may also be 0 */
@@ -70,13 +73,13 @@ struct ipx_packet
 
 struct ncp_request_header
 {
-	__u16 type __attribute__((packed));
-	__u8 sequence __attribute__((packed));
-	__u8 conn_low __attribute__((packed));
-	__u8 task __attribute__((packed));
-	__u8 conn_high __attribute__((packed));
-	__u8 function __attribute__((packed));
-	__u8 data[0] __attribute__((packed));
+	u_int16_t type __attribute__((packed));
+	u_int8_t sequence __attribute__((packed));
+	u_int8_t conn_low __attribute__((packed));
+	u_int8_t task __attribute__((packed));
+	u_int8_t conn_high __attribute__((packed));
+	u_int8_t function __attribute__((packed));
+	u_int8_t data[0] __attribute__((packed));
 };
 
 #define NCP_REPLY                (0x3333)
@@ -84,33 +87,33 @@ struct ncp_request_header
 
 struct ncp_reply_header
 {
-	__u16 type __attribute__((packed));
-	__u8 sequence __attribute__((packed));
-	__u8 conn_low __attribute__((packed));
-	__u8 task __attribute__((packed));
-	__u8 conn_high __attribute__((packed));
-	__u8 completion_code __attribute__((packed));
-	__u8 connection_state __attribute__((packed));
-	__u8 data[0] __attribute__((packed));
+	u_int16_t type __attribute__((packed));
+	u_int8_t sequence __attribute__((packed));
+	u_int8_t conn_low __attribute__((packed));
+	u_int8_t task __attribute__((packed));
+	u_int8_t conn_high __attribute__((packed));
+	u_int8_t completion_code __attribute__((packed));
+	u_int8_t connection_state __attribute__((packed));
+	u_int8_t data[0] __attribute__((packed));
 };
 
 #define NCP_BURST_PACKET         (0x7777)
 
 struct ncp_burst_header
 {
-	__u16 type __attribute__((packed));
-	__u8 system_flags __attribute__((packed));
-	__u8 stream_type __attribute__((packed));
-	__u32 source_conn __attribute__((packed));
-	__u32 dest_conn __attribute__((packed));
-	__u32 packet_sequence __attribute__((packed));
-	__u32 send_delay __attribute__((packed));
-	__u16 burst_sequence __attribute__((packed));
-	__u16 ack_sequence __attribute__((packed));
-	__u32 burst_length __attribute__((packed));
-	__u32 data_offset __attribute__((packed));
-	__u16 data_bytes __attribute__((packed));
-	__u16 missing_frags __attribute__((packed));
+	u_int16_t type __attribute__((packed));
+	u_int8_t system_flags __attribute__((packed));
+	u_int8_t stream_type __attribute__((packed));
+	u_int32_t source_conn __attribute__((packed));
+	u_int32_t dest_conn __attribute__((packed));
+	u_int32_t packet_sequence __attribute__((packed));
+	u_int32_t send_delay __attribute__((packed));
+	u_int16_t burst_sequence __attribute__((packed));
+	u_int16_t ack_sequence __attribute__((packed));
+	u_int32_t burst_length __attribute__((packed));
+	u_int32_t data_offset __attribute__((packed));
+	u_int16_t data_bytes __attribute__((packed));
+	u_int16_t missing_frags __attribute__((packed));
 };
 
 void handle_ipx(unsigned char *buf, int length, char *frame, int no);
@@ -130,10 +133,10 @@ int handle_burst(struct sockaddr_ipx *source,
  */
 typedef struct saps
 {
-	__u16 serverType __attribute__((packed));	/* NBO */
-	__u8 serverName[SAP_MAX_SERVER_NAME_LENGTH] __attribute__((packed));
+	u_int16_t serverType __attribute__((packed));	/* NBO */
+	u_int8_t serverName[SAP_MAX_SERVER_NAME_LENGTH] __attribute__((packed));
 	struct ipx_address serverAddress __attribute__((packed));
-	__u16 serverHops __attribute__((packed));	/* NBO */
+	u_int16_t serverHops __attribute__((packed));	/* NBO */
 }
 SAPS;
 
@@ -143,13 +146,13 @@ union sap_packet
 	unsigned short sapOperation;
 	struct sap_query
 	{
-		__u16 sapOperation __attribute__((packed));
-		__u16 serverType __attribute__((packed));
+		u_int16_t sapOperation __attribute__((packed));
+		u_int16_t serverType __attribute__((packed));
 	}
 	query;
 	struct sap_response
 	{
-		__u16 sapOperation __attribute__((packed));
+		u_int16_t sapOperation __attribute__((packed));
 		/* each SAP can has a max of SAP_MAX_SAPS_PER_PACKET packets */
 		SAPS sap[SAP_MAX_SAPS_PER_PACKET] __attribute__((packed));
 	}
@@ -174,13 +177,31 @@ print_sap(FILE * file, SAPS * sapp)
 void
 handle_ipx(unsigned char *buf, int length, char *frame, int no)
 {
-	struct ipx_packet *h = (struct ipx_packet *) buf;
+	struct ipx_packet *h;
 	struct sockaddr_ipx s_addr;
 	struct sockaddr_ipx d_addr;
 	union sap_packet *sappacket;
 	int hbo_dsock;		/* Host Byte Order of Destination SOCKet */
 	int hbo_sapop;		/* Host Byte Order of SAP OPeration */
 
+	if (frame && !strcmp(frame, "raweth")) {
+		int skip;
+		
+		if (((buf[12] << 8) | buf[13]) <= 0x5DC) {
+			skip = (buf[14] << 8) | buf[15];
+			switch (skip) {
+				case 0xFFFF: skip = 12; break;
+				case 0xAAAA: skip = 12 + 3 + 5; break;
+				default:     skip = 12 + 3; break;
+			}
+		} else
+			skip = 12 + 2;
+		if (length < skip)
+			return;	/* Whoops! */
+		length -= skip;
+		buf += skip;
+	}
+	h = (struct ipx_packet *)buf;
 	memset(&s_addr, 0, sizeof(s_addr));
 	memset(&d_addr, 0, sizeof(d_addr));
 
@@ -318,11 +339,11 @@ handle_burst(struct sockaddr_ipx *source,
 		struct ncp_burst_request
 		{
 			struct ncp_burst_header h __attribute__((packed));
-			__u32 function __attribute__((packed));
-			__u32 file_handle __attribute__((packed));
-			__u8 reserved[8] __attribute__((packed));
-			__u32 file_offset __attribute__((packed));
-			__u32 number_of_bytes __attribute__((packed));
+			u_int32_t function __attribute__((packed));
+			u_int32_t file_handle __attribute__((packed));
+			u_int8_t reserved[8] __attribute__((packed));
+			u_int32_t file_offset __attribute__((packed));
+			u_int32_t number_of_bytes __attribute__((packed));
 		}
 		*brq = (struct ncp_burst_request *) rq;
 
@@ -572,11 +593,11 @@ handle_ncp(struct sockaddr_ipx *source,
 					printf("Name Space: %d\n", p[1]);
 					printf("Open Create Mode: %x\n", p[2]);
 					printf("Search Attributes: %x\n",
-					       *(__u16 *) & (p[3]));
+					       *(u_int16_t *) & (p[3]));
 					printf("Return Information Mask: %x\n",
-					(unsigned int) (*(__u32 *) & (p[5])));
+					(unsigned int) (*(u_int32_t *) & (p[5])));
 					printf("Desired Access Rights: %x\n",
-					       *(__u16 *) & (p[9]));
+					       *(u_int16_t *) & (p[9]));
 					break;
 				}
 			case 2:
@@ -603,8 +624,8 @@ handle_ncp(struct sockaddr_ipx *source,
 			{
 				struct INPUT
 				{
-					__u16 proposed_max_size;
-					__u8 security_flag;
+					u_int16_t proposed_max_size;
+					u_int8_t security_flag;
 				}
 				*i = (struct INPUT *) data;
 
@@ -620,11 +641,11 @@ handle_ncp(struct sockaddr_ipx *source,
 			{
 				struct INPUT
 				{
-					__u32 local_conn_id;
-					__u32 local_max_packet_size;
-					__u16 local_target_socket;
-					__u32 local_max_send_size;
-					__u32 local_max_recv_size;
+					u_int32_t local_conn_id;
+					u_int32_t local_max_packet_size;
+					u_int16_t local_target_socket;
+					u_int32_t local_max_send_size;
+					u_int32_t local_max_recv_size;
 				}
 				*i = (struct INPUT *) data;
 
@@ -667,12 +688,12 @@ handle_ncp(struct sockaddr_ipx *source,
 					{
 						struct INPUT
 						{
-							__u8 subfunction_code;
-							__u32 fragger_handle;
-							__u32 max_fragment_size;
-							__u32 message_size;
-							__u32 fragment_flag;
-							__u32 verb;
+							u_int8_t subfunction_code;
+							u_int32_t fragger_handle;
+							u_int32_t max_fragment_size;
+							u_int32_t message_size;
+							u_int32_t fragment_flag;
+							u_int32_t verb;
 						}
 						*i = (struct INPUT *) data;
 						printf("Send NDS Fragment Request/Reply\n");
@@ -968,8 +989,8 @@ handle_ncp(struct sockaddr_ipx *source,
 					{
 						struct XDATA
 						{
-							__u8 new_directory_handle;
-							__u8 access_rights_mask;
+							u_int8_t new_directory_handle;
+							u_int8_t access_rights_mask;
 						}
 						*x = (struct XDATA *) data;
 						printf("new_directory_handle: %x\n",
@@ -988,9 +1009,9 @@ handle_ncp(struct sockaddr_ipx *source,
 				{
 					struct XDATA
 					{
-						__u16 accepted_max_size;
-						__u16 echo_socket;
-						__u8 security_flag;
+						u_int16_t accepted_max_size;
+						u_int16_t echo_socket;
+						u_int8_t security_flag;
 					}
 					*x = (struct XDATA *) data;
 					printf("accepted_max_size: %x\n",
@@ -1005,9 +1026,9 @@ handle_ncp(struct sockaddr_ipx *source,
 				{
 					struct XDATA
 					{
-						__u8 completion_code;
-						__u32 remote_target_id;
-						__u32 remote_max_packet_size;
+						u_int8_t completion_code;
+						u_int32_t remote_target_id;
+						u_int32_t remote_max_packet_size;
 					}
 					*x = (struct XDATA *) data;
 					printf("completion_code: %x\n",
@@ -1062,7 +1083,7 @@ handle_ncp(struct sockaddr_ipx *source,
 }
 
 
-void
+int
 main(int argc, char *argv[])
 {
 	unsigned char buf[16384];
@@ -1103,6 +1124,5 @@ main(int argc, char *argv[])
 		handle_ipx(packet, len, buf, i);
 		i += 1;
 	}
-
-	exit(0);
+	return 0;
 }
