@@ -82,33 +82,27 @@
 #endif
 
 /* set macros for MULTUNIT */
-#ifdef MUNIT8
-#define MULTUNITSIZE   8
+#if MULTUNITSIZE == 8
 typedef unsigned char MULTUNIT;
-#ifdef UNIT8
-#define MULTUNIT_SIZE_SAME
-#endif
-#else	/* not MUNIT8 */
-#ifdef MUNIT32
-#define MULTUNITSIZE   32
-typedef unsigned long MULTUNIT;
-#ifdef UNIT32
-#define MULTUNIT_SIZE_SAME
 #else
-/* #error is not portable, this has the same effect */
-#include "UNITSIZE cannot be smaller than MULTUNITSIZE"
+#if MULTUNITSIZE == 32
+typedef word32 MULTUNIT;
+#else
+#if MULTUNITSIZE == 16
+typedef word16 MULTUNIT;
+else
+#error "Unsupported MULTUNITSIZE"
+#endif
+#endif
+#endif
+
+#if UNITSIZE == MULTUNITSIZE
+#  define MULTUNIT_SIZE_SAME
+#else
+#if UNITSIZE < MULTUNITSIZE
+#  include "UNITSIZE cannot be smaller than MULTUNITSIZE"
+#endif
 #endif
-#else   /* assume MUNIT16 */
-#define MULTUNITSIZE   16
-typedef unsigned short MULTUNIT;
-#ifdef UNIT16
-#define MULTUNIT_SIZE_SAME
-#endif	/* UNIT16 */
-#ifdef UNIT8
-#include "UNITSIZE cannot be smaller than MULTUNITSIZE"
-#endif	/* UNIT8 */
-#endif	/* MUNIT32 */
-#endif	/* MUNIT8 */
 
 #define MULTUNIT_msb    ((MULTUNIT)1 << (MULTUNITSIZE-1)) /* msb of MULTUNIT */
 #define DMULTUNIT_msb   (1L << (2*MULTUNITSIZE-1))
@@ -116,8 +110,8 @@ typedef unsigned short MULTUNIT;
 #define MULTUNITs_perunit   (UNITSIZE/MULTUNITSIZE)
 
 
-void mp_smul (MULTUNIT *prod, MULTUNIT *multiplicand, MULTUNIT multiplier);
-void mp_dmul (unitptr prod, unitptr multiplicand, unitptr multiplier);
+static inline void mp_smul (MULTUNIT *prod, const MULTUNIT *multiplicand, MULTUNIT multiplier);
+static void mp_dmul (unitptr prod, const unit* multiplicand, const unit* multiplier);
 
 short global_precision = 0; /* units of precision for all routines */
 /*	global_precision is the unit precision last set by set_precision.
@@ -136,7 +130,7 @@ typedef unsigned long int ulint;
 
 #ifndef mp_addc
 boolean mp_addc
-	(register unitptr r1,register unitptr r2,register boolean carry)
+	(register unitptr r1, const unit* r2,register boolean carry)
 	/* multiprecision add with carry r2 to r1, result in r1 */
 	/* carry is incoming carry flag-- value should be 0 or 1 */
 {	register unit x;
@@ -162,7 +156,7 @@ boolean mp_addc
 
 #ifndef mp_subb
 boolean mp_subb
-	(register unitptr r1,register unitptr r2,register boolean borrow)
+	(register unitptr r1, const unit* r2,register boolean borrow)
 	/* multiprecision subtract with borrow, r2 from r1, result in r1 */
 	/* borrow is incoming borrow flag-- value should be 0 or 1 */
 {	register unit x;
@@ -208,6 +202,40 @@ boolean mp_rotate_left(register unitptr r1,register boolean carry)
 /************** end of primitives that should be in assembly *************/
 
 
+void mp_move_and_shift_left_bits(unit *dst, const unit *src, int bits) {
+	int precision;
+	int tmpbits;
+
+	precision = global_precision;
+	make_lsbptr(dst, precision);
+	make_lsbptr(src, precision);
+	for (tmpbits = bits / UNITSIZE; precision && tmpbits--; ) {
+		*dst = 0;
+		pre_higherunit(dst);
+		precision--;
+	}
+	bits &= (UNITSIZE - 1);
+	if (bits) {
+		unit carry = 0;
+		tmpbits = UNITSIZE - bits;
+		while (precision--) {
+			unit nextcarry;
+
+			nextcarry = *src >> tmpbits;
+			*dst = (*src << bits) | carry;
+			carry = nextcarry;
+			pre_higherunit(dst);
+			pre_higherunit(src);
+		}
+	} else {
+		while (precision--) {
+			*dst = *src;
+			pre_higherunit(dst);
+			pre_higherunit(src);
+		}
+	}
+}
+
 /*	The mp_shift_right_bits function is not called in any time-critical
 	situations in public-key cryptographic functions, so it doesn't
 	need to be coded in assembly language.
@@ -245,7 +273,7 @@ void mp_shift_right_bits(register unitptr r1,register short bits)
 
 
 #ifndef mp_compare
-short mp_compare(register unitptr r1,register unitptr r2)
+short mp_compare(const unit* r1, const unit* r2)
 /*	Compares multiprecision integers *r1, *r2, and returns:
 	-1 iff *r1 < *r2
 	 0 iff *r1 == *r2
@@ -320,13 +348,13 @@ void mp_init(register unitptr r, word16 value)
 	unitfill0( r, global_precision);
 	make_lsbptr(r,global_precision);
 	*post_higherunit(r) = value;
-#ifdef UNIT8
+#if UNITSIZE < 16
 	*post_higherunit(r) = value >> UNITSIZE;
 #endif
 }	/* mp_init */
 
 
-short significance(register unitptr r)
+short significance(const unit* r)
 	/* Returns number of significant units in r */
 {	register short precision;
 	precision = global_precision;
@@ -467,7 +495,7 @@ word16 mp_shortdiv(register unitptr quotient,
 
 
 int mp_mod(register unitptr remainder,
-	register unitptr dividend,register unitptr divisor)
+	const unit* dividend, const unit* divisor)
 	/* Unsigned divide, treats both operands as positive. */
 {	int bits;
 	short dprec;
@@ -487,7 +515,7 @@ int mp_mod(register unitptr remainder,
 } /* mp_mod */
 
 
-word16 mp_shortmod(register unitptr dividend,register word16 divisor)
+word16 mp_shortmod(const unit* dividend,register word16 divisor)
 /*	This function does a fast mod operation on a multiprecision dividend
 	using a short integer modulus returning a short integer remainder.
 	This is an unsigned divide.  It treats both operands as positive.
@@ -519,7 +547,7 @@ word16 mp_shortmod(register unitptr dividend,register word16 divisor)
 	/* We are skipping this code because it has a bug... */
 
 int mp_mult(register unitptr prod,
-	register unitptr multiplicand, register unitptr multiplier)
+	const unit *multiplicand, const unit *multiplier)
 	/*	Computes multiprecision prod = multiplicand * multiplier */
 {	/*	Uses interleaved comb multiply algorithm.
 		This improved multiply more than twice as fast as a Russian
@@ -602,7 +630,7 @@ int mp_mult(register unitptr prod,
 */
 
 int mp_mult(register unitptr prod,
-	register unitptr multiplicand,register unitptr multiplier)
+	const unit *multiplicand, const unit *multiplier)
 	/* Computes multiprecision prod = multiplicand * multiplier */
 {	/* Uses "Russian peasant" multiply algorithm. */
 	int bits;
@@ -658,9 +686,9 @@ static unsigned int tally_modsquares = 0;
 #ifdef PEASANT
 /* Conventional Russian peasant multiply with modulo algorithm. */
 
-static unitptr pmodulus = 0;	/* used only by mp_modmult */
+static const unit* pmodulus = 0;	/* used only by mp_modmult */
 
-int stage_peasant_modulus(unitptr n)
+int stage_peasant_modulus(const unit* n)
 /*	Must pass modulus to stage_modulus before calling modmult.
 	Assumes that global_precision has already been adjusted to the
 	size of the modulus, plus SLOP_BITS.
@@ -749,18 +777,18 @@ static unitptr moduli[UNITSIZE] = /* contains pointers into moduli_buf */
 {	0
 	,&moduli_buf[ 0][0], &moduli_buf[ 1][0], &moduli_buf[ 2][0], &moduli_buf[ 3][0],
 	 &moduli_buf[ 4][0], &moduli_buf[ 5][0], &moduli_buf[ 6][0]
-#ifndef UNIT8
+#if UNITSIZE > 8
 	,&moduli_buf[ 7][0]
 	,&moduli_buf[ 8][0], &moduli_buf[ 9][0], &moduli_buf[10][0], &moduli_buf[11][0], 
 	 &moduli_buf[12][0], &moduli_buf[13][0], &moduli_buf[14][0]
-#ifndef UNIT16	/* and not UNIT8 */
+#if UNITSIZE > 16
 	,&moduli_buf[15][0]
 	,&moduli_buf[16][0], &moduli_buf[17][0], &moduli_buf[18][0], &moduli_buf[19][0], 
 	 &moduli_buf[20][0], &moduli_buf[21][0], &moduli_buf[22][0], &moduli_buf[23][0], 
 	 &moduli_buf[24][0], &moduli_buf[25][0], &moduli_buf[26][0], &moduli_buf[27][0], 
 	 &moduli_buf[28][0], &moduli_buf[29][0], &moduli_buf[30][0]
-#endif	/* UNIT16 and UNIT8 not defined */
-#endif	/* UNIT8 not defined */
+#endif	/* UNITSIZE > 16 */
+#endif	/* UNITSIZE > 8 */
 };
 
 /*	To optimize msubs, need following 2 unit arrays, each filled
@@ -793,7 +821,7 @@ static void stage_mp_images(unitptr images[UNITSIZE],unitptr r)
 }	/* stage_mp_images */
 
 
-int stage_merritt_modulus(unitptr n)
+int stage_merritt_modulus(const unit* n)
 /*	Computes UNITSIZE images of modulus, each shifted left 1 more bit.
 	Before calling mp_modmult, you must first stage the moduli images by
 	calling stage_modulus.  n is the pointer to the modulus.
@@ -870,16 +898,16 @@ int merritt_modmult(register unitptr prod,
 	static unitptr mpd[UNITSIZE] =
 	{	 0,              &mpdbuf[ 0][0], &mpdbuf[ 1][0], &mpdbuf[ 2][0],
 		 &mpdbuf[ 3][0], &mpdbuf[ 4][0], &mpdbuf[ 5][0], &mpdbuf[ 6][0]
-#ifndef UNIT8
+#if UNITSIZE > 8
 		,&mpdbuf[ 7][0], &mpdbuf[ 8][0], &mpdbuf[ 9][0], &mpdbuf[10][0],
 		 &mpdbuf[11][0], &mpdbuf[12][0], &mpdbuf[13][0], &mpdbuf[14][0]
-#ifndef UNIT16	/* and not UNIT8 */
+#if UNITSIZE > 16
 		,&mpdbuf[15][0], &mpdbuf[16][0], &mpdbuf[17][0], &mpdbuf[18][0],
 		 &mpdbuf[19][0], &mpdbuf[20][0], &mpdbuf[21][0], &mpdbuf[22][0],
 		 &mpdbuf[23][0], &mpdbuf[24][0], &mpdbuf[25][0], &mpdbuf[26][0],
 		 &mpdbuf[27][0], &mpdbuf[28][0], &mpdbuf[29][0], &mpdbuf[30][0]
-#endif	/* UNIT16 and UNIT8 not defined */
-#endif	/* UNIT8 not defined */
+#endif	/* UNITSIZE > 16 */
+#endif	/* UNITSIZE > 8 */
 	};
 
 	/* Compute preshifted images of multiplicand, mod n: */
@@ -936,8 +964,8 @@ int merritt_modmult(register unitptr prod,
 				mp_sub(prod,moduli[i]);
 		*/
 
-#ifndef UNIT8
-#ifndef UNIT16  /* and not UNIT8 */
+#if UNITSIZE > 8
+#if UNITSIZE > 16
 		msubs(31);
 		msubs(30);
 		msubs(29);
@@ -954,7 +982,7 @@ int merritt_modmult(register unitptr prod,
 		msubs(18);
 		msubs(17);
 		msubs(16);
-#endif  /* not UNIT16 and not UNIT8 */
+#endif  /* UNITSIZE > 16 */
 		msubs(15);
 		msubs(14);
 		msubs(13);
@@ -963,13 +991,13 @@ int merritt_modmult(register unitptr prod,
 		msubs(10);
 		msubs(9);
 		msubs(8);
-#endif  /* not UNIT8 */
+#endif  /* UNITSIZE > 8 */
 		msubs(7);
 		msubs(6);
 		msubs(5);
-#ifndef UNIT32
+#if UNITSIZE < 32
 		msubs(4);
-#ifndef UNIT16
+#if UNITSIZE < 16
 		msubs(3);
 #endif
 #endif
@@ -985,8 +1013,8 @@ int merritt_modmult(register unitptr prod,
 		   if (*multiplier & power_of_2(i))
 				mp_add(prod,mpd[i]);
 		*/
-#ifndef UNIT8
-#ifndef UNIT16	/* and not UNIT8 */
+#if UNITSIZE > 8
+#if UNITSIZE > 16
 		sniffadd(31);
 		sniffadd(30);
 		sniffadd(29);
@@ -1003,7 +1031,7 @@ int merritt_modmult(register unitptr prod,
 		sniffadd(18);
 		sniffadd(17);
 		sniffadd(16);
-#endif	/* not UNIT16 and not UNIT8 */
+#endif	/* UNITSIZE > 16 */
 		sniffadd(15);
 		sniffadd(14);
 		sniffadd(13);
@@ -1012,7 +1040,7 @@ int merritt_modmult(register unitptr prod,
 		sniffadd(10);
 		sniffadd(9);
 		sniffadd(8);
-#endif	/* not UNIT8 */
+#endif	/* UNITSIZE > 8 */
 		sniffadd(7);
 		sniffadd(6);
 		sniffadd(5);
@@ -1035,8 +1063,8 @@ int merritt_modmult(register unitptr prod,
 				mp_sub(prod,moduli[i]); 
 		*/
 
-#ifndef UNIT8
-#ifndef UNIT16
+#if UNITSIZE > 8
+#if UNITSIZE > 16
 		msubs(5); 
 #endif
 		msubs(4);
@@ -1115,21 +1143,52 @@ short munit_prec;	/* global_precision expressed in MULTUNITs */
 */
 
 #ifndef mp_smul
-void mp_smul (MULTUNIT *prod, MULTUNIT *multiplicand, MULTUNIT multiplier)
+#if MULTUNITSIZE == 32 && defined(__i386__)
+static inline void mp_smul (MULTUNIT *prod, const MULTUNIT *multiplicand, MULTUNIT multiplier)
 {
-	short i;
-	unsigned long p, carry;
-
+	MULTUNIT carry;
+	int i;
+	
 	carry = 0;
-	for (i=0; i<munit_prec; ++i)
-	{	p = (unsigned long)multiplier * *post_higherunit(multiplicand);
-		p += *prod + carry;
-		*post_higherunit(prod) = (MULTUNIT)p;
-		carry = p >> MULTUNITSIZE;
+	for (i=munit_prec; i!=0; --i)
+	{
+		unsigned int dummy;
+		
+		asm volatile (
+			"mull %6\n\t"
+			"addl %3,%2\n\t"
+			"adcl $0,%0\n\t"
+			"addl %2,(%1)\n\t"
+			"adcl $0,%0\n\t"
+			:
+			"=d"(carry), "=D"(prod), "=a"(dummy) :
+			"c"(carry), "1"(prod), "2"(*multiplicand), "m"(multiplier)
+		);
+		post_higherunit(multiplicand);
+		post_higherunit(prod);
 	}
 	/* Add carry to the next higher word of product / dividend */
 	*prod += (MULTUNIT)carry;
 }	/* mp_smul */
+#else
+static inline void mp_smul (MULTUNIT *prod, const MULTUNIT *multiplicand, MULTUNIT multiplier)
+{
+	unsigned long lmult = (unsigned long)multiplier;
+	unsigned long carry;
+	int i;
+	
+	carry = 0;
+	for (i=munit_prec; i!=0; --i)
+	{
+		carry += lmult * *post_higherunit(multiplicand);
+		carry += *prod;
+		*post_higherunit(prod) = (MULTUNIT)carry;
+		carry = carry >> MULTUNITSIZE;
+	}
+	/* Add carry to the next higher word of product / dividend */
+	*prod += (MULTUNIT)carry;
+}	/* mp_smul */
+#endif
 #endif
 
 /*	mp_dmul is a double-precision multiply multiplicand times multiplier,
@@ -1138,18 +1197,18 @@ void mp_smul (MULTUNIT *prod, MULTUNIT *multiplicand, MULTUNIT multiplier)
 	pointing at a 20-word buffer.
 */
 #ifndef mp_dmul
-void mp_dmul (unitptr prod, unitptr multiplicand, unitptr multiplier)
+void mp_dmul (unitptr prod, const unit* multiplicand, const unit* multiplier)
 {
 	register	int i;
-	register	MULTUNIT *p_multiplicand, *p_multiplier;
+	const		MULTUNIT *p_multiplicand, *p_multiplier;
 	register	MULTUNIT *prodp;
 
 
 	unitfill0(prod,global_precision*2);	/* Pre-zero prod */
 	/* Calculate precision in units of MULTUNIT */
 	munit_prec = global_precision * UNITSIZE / MULTUNITSIZE;
-	p_multiplicand = (MULTUNIT *)multiplicand;
-	p_multiplier = (MULTUNIT *)multiplier;
+	p_multiplicand = (const MULTUNIT *)multiplicand;
+	p_multiplier = (const MULTUNIT *)multiplier;
 	prodp = (MULTUNIT *)prod;
 	make_lsbptr(p_multiplicand,munit_prec);
 	make_lsbptr(p_multiplier,munit_prec);
@@ -1190,7 +1249,7 @@ static short nbitsModUNITSIZE;
 	Assumes that global_precision has already been adjusted to the
 	size of the modulus, plus SLOP_BITS.
 */
-int stage_upton_modulus(unitptr n)
+int stage_upton_modulus(const unit* n)
 {
 	mp_move(modulus, n);
 	mp_recip(reciprocal, modulus);
@@ -1344,7 +1403,7 @@ MULTUNIT mp_quo_digit (MULTUNIT *dividend);
 */
 #ifndef mp_musubb
 boolean mp_musubb
-	(register MULTUNIT* r1,register MULTUNIT* r2,register boolean borrow)
+	(register MULTUNIT* r1,const MULTUNIT* r2,register boolean borrow)
 	/* multiprecision subtract of MULTUNITs with borrow, r2 from r1,
 	** result in r1 */
 	/* borrow is incoming borrow flag-- value should be 0 or 1 */
@@ -1473,7 +1532,7 @@ MULTUNIT mp_quo_digit (MULTUNIT *dividend) {
 	stage_smith_modulus() is aliased to stage_modulus().
 */
 
-int stage_smith_modulus(unitptr n_modulus)
+int stage_smith_modulus(const unit* n_modulus)
 {
 	int   original_precision;
 	int   sigmod;		/* significant units in modulus */
@@ -1608,7 +1667,7 @@ smith_modmult(unitptr prod, unitptr multiplicand, unitptr multiplier)
 					unmake_lsbptr(dmp, orig_precision * 
 						   MULTUNITs_perunit);
 					if (mp_musubb(dmp,
-						   (MULTUNIT *) modulus, 0))
+						   (const MULTUNIT *) modulus, 0))
 						(*dmph)  --;
 				}
 			}
@@ -1654,7 +1713,7 @@ void smith_burn(void)
 #endif	/* SMITH */
 
 
-int countbits(unitptr r)
+int countbits(const unit* r)
 	/* Returns number of significant bits in r */
 {	int bits;
 	short prec;
@@ -1670,7 +1729,7 @@ const char *copyright_notice(void)
 
 
 int mp_modexp(register unitptr expout,register unitptr expin,
-	register unitptr exponent,register unitptr modulus)
+	register unitptr exponent,const unit* modulus)
 {	/*	Russian peasant combined exponentiation/modulo algorithm.
 		Calls modmult instead of mult.
 		Computes:  expout = (expin**exponent) mod modulus
