@@ -15,6 +15,14 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    
+    Revision history:
+    
+	0.00  1997			Arne de Bruijn
+						Initial release.
+	0.01  1997			Petr Vandrovec <vandrove@vc.cvut.cz>
+						Remove shuffle.
+
 */
 
 #include "config.h"
