@@ -1,4 +1,46 @@
-/* Library extensions for nwdir.c */
+/* 
+    nwcalls.c - Connection oriented calls
+    Copyright (C) 1999, 2000  Petr Vandrovec
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+    Revision history:
+
+	0.00  1998, December 1		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Initial release.
+
+	1.00  1999, November 20		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Added copyright and license.
+
+	1.01  2000, January 16		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Changes for 32bit uids.
+		
+	1.02  2000, May 1		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Added NWGetFileServerUTCTime.
+
+	1.03  2000, May 7		Petr Vandrovec <vandrove@vc.cvut.cz>
+		NWGetFileServerUTCTime moved to nwtime.c.
+
+	1.04  2000, May 27		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Make NWParsePath return ERR_NULL_POINTER if path==NULL.
+
+	1.05  2000, June 1		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Added NWGetBinderyAccessLevel.
+
+ */
+
 #include "config.h"
 
 #include <errno.h>
@@ -6,6 +48,7 @@
 #include <stdlib.h>
 
 #include <ncp/nwcalls.h>
+#include <ncp/nwnet.h>
 #include "ncplib_i.h"
 #include "ncpcode.h"
 
@@ -142,6 +185,27 @@ NWCCODE NWGetObjectID(NWCONN_HANDLE conn, char* name, NWObjectType type, NWObjec
 	return 0;
 }
 
+NWCCODE NWGetBinderyAccessLevel(NWCONN_HANDLE conn,
+		nuint8* level, NWObjectID* id) {
+	nuint8 reply[16];
+	NW_FRAGMENT rp;
+	NWCCODE err;
+	
+	rp.fragAddr.rw = reply;
+	rp.fragSize = sizeof(reply);
+	
+	err = NWRequestSimple(conn, NCPC_SFN(0x17, 0x46), NULL, 0, &rp);
+	if (err)
+		return err;
+	if (rp.fragSize < 5)
+		return NWE_INVALID_NCP_PACKET_LENGTH;
+	if (level)
+		*level = BVAL(reply, 0);
+	if (id)
+		*id = DVAL_HL(reply, 1);
+	return 0;
+} 
+
 NWCCODE NWGetNSEntryInfo(NWCONN_HANDLE conn, nuint dirHandle, const char* path, 
 	nuint srcNS, nuint dstNS, nuint16 attr, nuint32 rim, NW_ENTRY_INFO* info) {
 	unsigned char buff[1024];
@@ -163,6 +227,9 @@ NWCCODE NWParsePath(const char* path, char* server, NWCONN_HANDLE* conn,
 	char tmp[1000];
 	char* ptr;
 
+	if (!path)
+		return ERR_NULL_POINTER;
+
 	result = ncp_open_mount(path, &cn);
 	if (result) {
 		if (volume)
@@ -445,7 +512,7 @@ static NWCCODE NCPGetObjectConnectionList(NWCONN_HANDLE conn, NWCONN_NUM first,
 	return 0;
 }
 
-static NWCCODE NCPGetConnListFromObject(NWCONN_HANDLE conn, u_int32_t objID,
+static NWCCODE NCPGetConnListFromObject(NWCONN_HANDLE conn, NWObjectID objID,
 		NWCONN_NUM firstConn, size_t* conns, NWCONN_NUM* connlist) {
 	nuint8 rq_array[4+4];
 	nuint8 rp_array[2+4*125]; /* 125 is magic NWServer constant... */
@@ -473,7 +540,7 @@ static NWCCODE NCPGetConnListFromObject(NWCONN_HANDLE conn, u_int32_t objID,
 	}
 	if (nconns > 125)
 		nconns = 125; /* ugh... Houston, we have a problem... */
-	if (conns) {
+	if (connlist) {
 		size_t i;
 		
 		for (i = 0; i < nconns; i++) {
@@ -484,8 +551,7 @@ static NWCCODE NCPGetConnListFromObject(NWCONN_HANDLE conn, u_int32_t objID,
 		*conns = nconns;
 	return 0;
 }
-		
-		
+
 NWCCODE NWGetObjectConnectionNumbers(NWCONN_HANDLE conn, const char* name,
 		nuint16 type, size_t* numConns, NWCONN_NUM* conns,
 		size_t maxConns) {
@@ -516,7 +582,7 @@ NWCCODE NWGetObjectConnectionNumbers(NWCONN_HANDLE conn, const char* name,
 	return 0;	
 }
 
-NWCCODE NWGetConnListFromObject(NWCONN_HANDLE conn, u_int32_t objID,
+NWCCODE NWGetConnListFromObject(NWCONN_HANDLE conn, NWObjectID objID,
 		NWCONN_NUM firstConn, size_t* conns, NWCONN_NUM* connlist) {
 	char name[NCP_BINDERY_NAME_LEN+1];
 	nuint16 type;
@@ -774,20 +840,24 @@ NWCCODE NWCCGetConnInfo(NWCONN_HANDLE conn, nuint info, size_t len,
 			}
 		case NWCC_INFO_USER_ID:
 			{
-				NWCONN_NUM cn;
-				NWCCODE err;
-				struct ncp_bindery_object obj;
-				
-				err = NWGetConnectionNumber(conn, &cn);
-				if (err)
-					return err;
-				err = ncp_get_stations_logged_info(conn, conn->i.connection, &obj, NULL);
-				if (err)
-					return err;
+				if (!conn->user_id_valid) {
+					NWCCODE err;
+					struct ncp_bindery_object obj;
+					NWCONN_NUM cn;
+
+					err = NWGetConnectionNumber(conn, &cn);
+					if (err)
+						return err;
+					err = ncp_get_stations_logged_info(conn, cn, &obj, NULL);
+					if (err)
+						return err;
+					conn->user_id = obj.object_id;
+					conn->user_id_valid = 1;
+				}
 				if (len != sizeof(NWObjectID))
 					return NWE_BUFFER_OVERFLOW;
-				*(NWObjectID*)buffer = obj.object_id;
-				return err;
+				*(NWObjectID*)buffer = conn->user_id;
+				return 0;
 			}
 		case NWCC_INFO_SERVER_NAME:
 			return NWGetFileServerNameInt(conn, len, buffer);
@@ -812,15 +882,15 @@ NWCCODE NWCCGetConnInfo(NWCONN_HANDLE conn, nuint info, size_t len,
 				NWCCTranAddr* p = buffer;
 				if (len < sizeof(NWCCTranAddr))
 					return NWE_BUFFER_OVERFLOW;
-				switch (conn->i.addr.any.sa_family) {
+				switch (conn->addr.any.sa_family) {
 #ifdef CONFIG_NATIVE_IP
 					case AF_INET:
 						if (p->len < 6)
 							return NWE_BUFFER_OVERFLOW;
 						p->len = 6;
 						p->type = NT_UDP;
-						memcpy(((char*)(p->buffer))+2, &conn->i.addr.inet.sin_addr.s_addr, 4);
-						memcpy(p->buffer, &conn->i.addr.inet.sin_port, 2);
+						memcpy(((char*)(p->buffer))+2, &conn->addr.inet.sin_addr.s_addr, 4);
+						memcpy(p->buffer, &conn->addr.inet.sin_port, 2);
 						break;
 #endif
 #ifdef CONFIG_NATIVE_IPX
@@ -829,9 +899,9 @@ NWCCODE NWCCGetConnInfo(NWCONN_HANDLE conn, nuint info, size_t len,
 							return NWE_BUFFER_OVERFLOW;
 						p->len = 12;
 						p->type = NT_IPX;
-						memcpy(p->buffer, &conn->i.addr.ipx.sipx_network, 4);
-						memcpy(((char*)(p->buffer))+4, conn->i.addr.ipx.sipx_node, 6);
-						memcpy(((char*)(p->buffer))+10, &conn->i.addr.ipx.sipx_port, 2);
+						memcpy(p->buffer, &conn->addr.ipx.sipx_network, 4);
+						memcpy(((char*)(p->buffer))+4, conn->addr.ipx.sipx_node, 6);
+						memcpy(((char*)(p->buffer))+10, &conn->addr.ipx.sipx_port, 2);
 						break;
 #endif
 					default:
