@@ -1,15 +1,42 @@
-/****************************************************************************
- *  ncopy.c
- *
- *  Copy file on a Netware server without Network Traffic
- *
- *  Copyright (C) 1996 by Brian Reid and Tom Henderson.
- * 
- *   Send bug reports for ncopy to "breid@tim.com"
- *
- *  Still to do: support recursive copy with two arguments
- *  Both must be directories. (similar to rcp -r)
- *
+/*
+    ncopy.c - Copy file on a Netware server without Network Traffic
+    Copyright (C) 1996 by Brian Reid and Tom Henderson.
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+    Send bug reports for ncopy to "breid@tim.com"
+ 
+    Still to do: support recursive copy with two arguments
+    Both must be directories. (similar to rcp -r)
+
+
+    Revision history:
+
+	0.01  1996			Brian Reid and Tom Henderson
+		Initial revision.
+
+	0.02  1999			Petr Vandrovec <vandrove@vc.cvut.cz>
+		Copy trustees and other Netware informations.
+
+	1.00  1999, November 20		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Added license.
+		
+	1.01  1999, November 21		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Copy MAC resource fork.
+
  */
 
 #include "config.h"
@@ -36,9 +63,11 @@
  * Globals:
  *
  */
-const char *VersionStr = "0.2";
+const char *VersionStr = "1.01";
 static char *ProgramName;
 
+#define RSRCSUBDIR	".rsrc/"
+
 /* (initialized) command options  */
 
 static int optVersion = 0;		/* -V TRUE if just want version */
@@ -54,8 +83,10 @@ static int optOwner = 0;		/* -pp */
 static int optFlags = 0;		/* -p */
 static int optTimes = 0;		/* -p too... Hack source... */
 static int optTrustees = 0;		/* -t */
+static int optMAC = 0;			/* -m */
+static int optMACinRsrc = 0;		/* -M */
 static int BlocksCopied = 0;		/* Number of blocks copied */
-static int MaxNcopyRetries = 25;	/* Maximum number of times to retry a failed
+static unsigned int MaxNcopyRetries = 25;	/* Maximum number of times to retry a failed
 				   copy before giving up  */
 
 /* Globals needed for signal handlers */
@@ -75,8 +106,8 @@ static void
 usage(void)
 {
 	fprintf(stderr, _("usage: %s [-V]\n"), ProgramName);
-	fprintf(stderr, _("       %s [-vnppt] [-s amt] sourcefile destinationfile|directory\n"), ProgramName);
-	fprintf(stderr, _("       %s [-vnppt] [-s amt] sourcefile [...] directory\n"), ProgramName);
+	fprintf(stderr, _("       %s [-vmMnppt] [-s amt] sourcefile destinationfile|directory\n"), ProgramName);
+	fprintf(stderr, _("       %s [-vmMnppt] [-s amt] sourcefile [...] directory\n"), ProgramName);
 }
 
 /****************************************************************************
@@ -128,7 +159,7 @@ handleOptions(const int argc, char *const argv[])
 {
 	int opt;
 
-	while ((opt = getopt(argc, argv, "vVns:pt")) != EOF)
+	while ((opt = getopt(argc, argv, "vVns:ptmM")) != EOF)
 	{
 		switch (opt)
 		{
@@ -165,6 +196,14 @@ handleOptions(const int argc, char *const argv[])
 		case 't':
 			optTrustees = 1;
 			break;
+
+		case 'm':	/* copy MAC resource fork */
+			optMAC = 1;
+			break;
+			
+		case 'M':	/* MAC resource fork in .rsrc subdirectory */
+			optMACinRsrc = 1;
+			break;
 			
 		default:	/* invalid options or options without required arguments */
 			return 1;
@@ -228,80 +267,151 @@ int xlatid(NWCONN_HANDLE scon, NWObjectID sID,
 	return 1;
 }
 
-/****************************************************************************
- * Does a regular buffered file copy.
- * This is used if we cannot use the Netware file copy.
- */
-static int
-normalFileCopy(int fdIn, int fdOut,
-	       const char *InputFile, const char *OutputFile,
-	       char *Buffer, int BufferSize)
-{
-	long fileSize = 0;
-	long filePos = 0;
-
-	if (optVerbose) {
-		/* we need fi;e size only for verbose... */
-		fileSize = lseek(fdIn, 0, SEEK_END);
-		if (fileSize == -1)
-			fileSize = 0;
-		lseek(fdIn, 0, SEEK_SET);
-		if (fileSize)
-			printf(_("Normal copy: %s -> %s %5.1f%%"), InputFile, OutputFile, 0.0);
-		else
-			printf(_("Normal copy: %s -> %s %ld"), InputFile, OutputFile, 0l);
-		fflush(stdout);
-	}
-	while (1)
-	{
-		int currentMove;
-		int writeAmt;
-		do {
-			currentMove = read(fdIn, Buffer, BufferSize);
-		} while ((currentMove == -1) && ((errno == EINTR) || (errno == EAGAIN) || (errno == EWOULDBLOCK)));
-		if (currentMove < 0) {
-			fprintf(stderr, _("%s: Error reading %s, %s\n"), ProgramName, InputFile,
-				strerror(errno));
-			return 1;
-		}
-		writeAmt = write(fdOut, Buffer, currentMove);
-		if (writeAmt < 0)
-		{
-			fprintf(stderr, _("%s: Error writing %s, %s\n"), ProgramName, OutputFile,
-				strerror(errno));
-			return 1;
-		}
-		if (!currentMove)
-			break;
-		if (writeAmt != currentMove) {
-			fprintf(stderr, _("%s: Out of space on destination device writing %s\n"),
-				ProgramName, OutputFile);
-			return 1;
-		}
-		if (optVerbose)
-		{
-			filePos += currentMove;
-			if (fileSize >= filePos)
-				printf(_("\rNormal copy: %s -> %s %5.1f%%"), InputFile, OutputFile, (((float)filePos) / fileSize) * 100.0);
-			else
-				printf(_("\rNormal copy: %s -> %s %ld"), InputFile, OutputFile, filePos);
-			fflush(stdout);
-		}
-	}
-	if (optVerbose)
-		printf("\n");
-	if (optOwner || optFlags || optTimes)
-		fprintf(stderr, _("Cannot preserve attributes for %s - not Netware\n"), OutputFile);
-	return 0;
-}
-
 /****************************************************************************
  *  Interfaces with the ncplib to do the netware copy of the file.
  */
+static int
+copyNWNW(
+		const char* header,
+		int sfd,
+		NWCONN_HANDLE scon,
+		const u_int8_t* sfhandle,
+		const char* source,
+		int dfd,
+		NWCONN_HANDLE dcon,
+		const u_int8_t* dfhandle,
+		const char* destination) {
+	ncp_off64_t totalSize;
+	ncp_off64_t sourceOff;
+	NWCCODE err;
+	unsigned int retryCount;
+	int failed;
+	int ncopy = scon && (scon == dcon);
+	int dolf = optVerbose;
+
+	if (scon) {
+		err = ncp_get_file_size(scon, sfhandle, &totalSize);
+		if (err)
+			totalSize = 0;
+	} else {
+		struct stat stats;
+		
+		err = fstat(sfd, &stats);
+		if (err)
+			totalSize = 0;
+		else
+			totalSize = stats.st_size;
+	}
+
+	sourceOff = 0;
+	retryCount = 0;
+
+	if (optVerbose)	{
+		if (totalSize)
+			printf(_("%s: %s -> %s %5.1f%%"), header, source, destination, 0.0);
+		else
+			printf(_("%s: %s -> %s %lld"), header, source, destination, 0LL);
+		fflush(stdout);
+	}
+	failed = -1;
+	while (retryCount < MaxNcopyRetries) {
+		if (ncopy) {
+			u_int32_t amountCopied;
+
+			/* If we are being nice and we've copied enough blocks, go to sleep */
+			if (optNice)
+			{
+				if (BlocksCopied == optNiceFactor)
+				{
+					sleep(NiceSleepTime);
+					BlocksCopied = 0;
+				} else
+					++BlocksCopied;
+			}
+			err = ncp_copy_file(dcon, sfhandle,
+					    dfhandle, sourceOff, 
+					    sourceOff,
+					    CopyBlockSize, &amountCopied);
+			if (err) {
+				/* In my testing this only happens when you run out of space 
+			   	on the server.
+			   	Netware seems to wait a bit before reporting space recently
+			   	free'd.  I will just wait a bit before bombin */
+				sleep(1);	/* Sleep for a second and try again */
+				retryCount++;
+			} else {
+				if (!amountCopied) {
+					ncopy = 0;
+				}
+				sourceOff += amountCopied;
+			}
+		} else {
+			u_int8_t buffer[49152];
+			long bytes;
+			long written;
+			
+			if (scon)
+				bytes = ncp_read(scon, sfhandle, sourceOff, sizeof(buffer), buffer);
+			else
+				bytes = read(sfd, buffer, sizeof(buffer));
+			if (bytes < 0) {
+				dolf = 0;
+				if (optVerbose)
+					putchar('\n');
+				fprintf(stderr, _("%s: Cannot read %s\n"),
+					ProgramName,
+					source);
+				break;
+			}
+			if (bytes == 0) {
+				/* whole file? */
+				if (sourceOff >= totalSize) {
+					failed = 0;
+					break;
+				}
+				sleep(1);
+				retryCount++;
+			} else {
+				if (dcon)
+					written = ncp_write(dcon, dfhandle, sourceOff, bytes, buffer);
+				else
+					written = write(dfd, buffer, bytes);
+				if (written != bytes) {
+					dolf = 0;
+					if (optVerbose)
+						putchar('\n');
+					fprintf(stderr, _("%s: Cannot write %s\n"),
+						ProgramName,
+						destination);
+					break;
+				}
+				sourceOff += bytes;
+			}
+		}
+		if (optVerbose)
+		{
+			putchar('\r');
+			if (totalSize && (totalSize >= sourceOff))
+				printf(_("%s: %s -> %s %5.1f%%"), header, source, destination,
+			       		((float) sourceOff) / totalSize * 100.0);
+			else
+				printf(_("%s: %s -> %s %lld"), header, source, destination,
+					sourceOff);
+			if (retryCount)
+				printf(_("  %d retries"), retryCount);
+			fflush(stdout);
+		}
+	}
+	if (dolf)
+		putchar('\n');
+	return failed;
+}
+
 static int
 netwareCopyFile(int fdIn, int fdOut,
 		const char *source,
-		const char *destination)
+		const char *destination,
+		mode_t sourceMode)
 {
 	NWCCODE err;
 	struct nw_info_struct2 sfi, dfi;
@@ -309,21 +419,39 @@ netwareCopyFile(int fdIn, int fdOut,
 	u_int8_t dfhandle[6];
 	NWCONN_HANDLE scon, dcon;
 	struct NWCCRootEntry sentry, dentry;
-	u_int32_t amountCopied;
-	u_int32_t amtLeft;
-	u_int32_t totalSize;
-	u_int32_t sourceOff;
-	u_int32_t thisMove;
 	int retValue = 2;
-	int retryCount;
 	int ncopy = 0;
 
 	err = ncp_open_fd(fdIn, &scon);
 	if (err) {
 		dfprintf(stderr, _("%s: Unable to open source file: %s\n"),
 			ProgramName, strnwerror(err));
-		return retValue;
+		scon = NULL;
+		goto srcDetermined;
 	}
+	err = NWCCGetConnInfo(scon, NWCC_INFO_ROOT_ENTRY, 
+		sizeof(sentry), &sentry);
+	if (err) {
+		dfprintf(stderr, _("%s: Unable to stat source file: %s\n"),
+			ProgramName, strnwerror(err));
+		ncp_close(scon);
+		scon = NULL;
+		goto srcDetermined;
+	}
+	err = ncp_ns_open_create_entry(scon, NW_NS_DOS, SA_ALL,
+		1, sentry.volume, sentry.dirEnt, NULL, 0, 
+		-1, OC_MODE_OPEN, 0, AR_READ_ONLY, 
+		RIM_ALL,
+		&sfi, sizeof(sfi), NULL, NULL, sfhandle);
+	if (err) {
+		dfprintf(stderr, _("%s: Unable to open source file: %s\n"),
+			ProgramName, strnwerror(err));
+		ncp_close(scon);
+		scon = NULL;
+		goto srcDetermined;
+	}
+
+srcDetermined:;
 	{
 		/* Grrr... All kernels since 2.1.43 up to 2.2.10 & 2.3.9 corrupts 
 		           DosDirNum on file open :-( stat() fixes it, fstat does not */
@@ -334,9 +462,33 @@ netwareCopyFile(int fdIn, int fdOut,
 	if (err) {
 		dfprintf(stderr, _("%s: Unable to open output file: %s\n"),
 			ProgramName, strnwerror(err));
-		goto cscon;
+		dcon = NULL;
+		goto dstDetermined;
 	}
-	{
+	err = NWCCGetConnInfo(dcon, NWCC_INFO_ROOT_ENTRY, 
+		sizeof(dentry), &dentry);
+	if (err) {
+		dfprintf(stderr, _("%s: Unable to stat destination file: %s\n"),
+			ProgramName, strnwerror(err));
+		ncp_close(dcon);
+		dcon = NULL;
+		goto dstDetermined;
+	}
+	err = ncp_ns_open_create_entry(dcon, NW_NS_DOS, SA_ALL,
+		1, dentry.volume, dentry.dirEnt, NULL, 0,
+		-1, OC_MODE_OPEN, 0, AR_WRITE_ONLY,
+		0,
+		&dfi, sizeof(dfi), NULL, NULL, dfhandle);
+	if (err) {
+		dfprintf(stderr, _("%s: Unable to open destination file: %s\n"),
+			ProgramName, strnwerror(err));
+		ncp_close(dcon);
+		dcon = NULL;
+		goto dstDetermined;
+	}
+
+dstDetermined:;
+	if (scon && dcon) {	
 		NWCONN_NUM snum, dnum;
 		
 		if (!NWGetConnectionNumber(scon, &snum) 
@@ -363,170 +515,179 @@ netwareCopyFile(int fdIn, int fdOut,
 					ncopy = !strcmp(saddr, daddr);
 				}
 			}
+			if (ncopy) {
+				ncp_close(scon);
+				scon = dcon;
+			}
 		}
 	}
-
-	err = NWCCGetConnInfo(scon, NWCC_INFO_ROOT_ENTRY, 
-		sizeof(sentry), &sentry);
-	if (err) {
-		dfprintf(stderr, _("%s: Unable to stat source file: %s\n"),
-			ProgramName, strnwerror(err));
-		goto cdcon;
-	}
-	err = ncp_ns_open_create_entry(scon, NW_NS_DOS, 0x8006,
-		1, sentry.volume, sentry.dirEnt, NULL, 0, 
-		-1, OC_MODE_OPEN, 0, AR_READ_ONLY, 
-		RIM_ALL,
-		&sfi, sizeof(sfi), NULL, NULL, sfhandle);
-	if (err) {
-		dfprintf(stderr, _("%s: Unable to open source file: %s\n"),
-			ProgramName, strnwerror(err));
-		goto cdcon;
-	}
-
-	err = NWCCGetConnInfo(dcon, NWCC_INFO_ROOT_ENTRY, 
-		sizeof(dentry), &dentry);
-	if (err) {
-		dfprintf(stderr, _("%s: Unable to stat destination file: %s\n"),
-			ProgramName, strnwerror(err));
-		goto csfi;
-	}
-	err = ncp_ns_open_create_entry(dcon, NW_NS_DOS, 0x8006,
-		1, dentry.volume, dentry.dirEnt, NULL, 0,
-		-1, OC_MODE_OPEN, 0, AR_WRITE_ONLY,
-		0,
-		&dfi, sizeof(dfi), NULL, NULL, dfhandle);
-	if (err) {
-		dfprintf(stderr, _("%s: Unable to open destination file: %s\n"),
-			ProgramName, strnwerror(err));
-		goto csfi;
-	}
-
-	if (ncopy) {
-		ncp_close(scon);
-		scon = dcon;
-	}
 	/* Set globals in case a signal happens while copying */
 	CurrentConn = dcon;
 	CurrentFile = dfhandle;
 
-	retValue = 0;
-	if (optVerbose)
-	{
-		printf(_("NetWare copy: %s -> %s %5.1f%%"), source, destination, 0.0);
-		fflush(stdout);
-	}
-	/* The main copy loop.  */
+	retValue = copyNWNW(_("NetWare copy"),
+			    fdIn, scon, sfhandle, source,
+			    fdOut, dcon, dfhandle, destination);
 
-	amtLeft = totalSize = sfi.DataSize;
-	sourceOff = 0;
-	retryCount = 0;
-
-	/* printf("AmtLeft: %d\n", amtLeft); */
-
-	while (amtLeft && retryCount < MaxNcopyRetries)
-	{
-		int ncopyRetValue;
-			
-		if (ncopy) {
-			if (amtLeft > CopyBlockSize)
-				thisMove = CopyBlockSize;
-			else
-				thisMove = amtLeft;
-			/* If we are being nice and we've copied enough blocks, go to sleep */
-			if (optNice)
-			{
-				if (BlocksCopied == optNiceFactor)
-				{
-					sleep(NiceSleepTime);
-					BlocksCopied = 0;
-				} else
-					++BlocksCopied;
-			}
-			ncopyRetValue = ncp_copy_file(dcon, sfhandle,
-						      dfhandle, sourceOff, 
-						      sourceOff,
-						      thisMove, &amountCopied);
-			if (ncopyRetValue != 0)
-			{
-				/* In my testing this only happens when you run out of space 
-			   	on the server.
-			   	Netware seems to wait a bit before reporting space recently
-			   	free'd.  I will just wait a bit before bombin */
-				sleep(1);	/* Sleep for a second and try again */
-				retryCount++;
-				amountCopied = thisMove = 0;
-			}
-			if (amountCopied != thisMove)
-			{
-				fprintf(stderr, _("%s: Warning, amountCopied (%u) != thisMove (%u)\n"),
-					ProgramName, (unsigned int) amountCopied, (unsigned int) thisMove);
-			}
-			amtLeft -= amountCopied;
-			sourceOff += amountCopied;
-#ifdef NCOPY_DEBUG
-			fprintf(stderr, _("Copied %u (actual %u)\n"), (unsigned int) thisMove,
-				(unsigned int) amountCopied);
-#endif
-		} else {
-			u_int8_t buffer[49152];
-			long bytes;
-			long written;
-			
-			if (amtLeft > 49152)
-				thisMove = 49152;
-			else
-				thisMove = amtLeft;
-			bytes = ncp_read(scon, sfhandle, sourceOff, thisMove, buffer);
-			if (bytes < 0) {
-				fprintf(stderr, _("%s: Cannot read %s\n"),
-					ProgramName,
-					source);
-				break;
-			}
-			if (bytes == 0) {
-				fprintf(stderr, _("%s: Unexpected end of %s\n"),
-					ProgramName,
-					source);
-				break;
-			}
-			written = ncp_write(dcon, dfhandle, sourceOff, bytes, buffer);
-			if (written != bytes) {
-				fprintf(stderr, _("%s: Cannot write %s\n"),
-					ProgramName,
-					destination);
-				break;
-			}
-			amtLeft -= bytes;
-			sourceOff += bytes;
-		}
-		if (optVerbose)
+	if (dcon) {
+		if (ncp_close_file(dcon, dfhandle) != 0)
 		{
-			putchar('\r');
-			printf(_("NetWare copy: %s -> %s %5.1f%%"), source, destination,
-		       		(float) (totalSize - amtLeft) / totalSize * 100.0);
-			if (retryCount)
-				printf(_("  %d retries"), retryCount);
-			fflush(stdout);
+			fprintf(stderr, _("%s: Close failed for %s\n"), ProgramName, destination);
+			retValue = 1;
+		}
+		CurrentFile = NULL;
+	}
+	if (scon) {
+		if (ncp_close_file(scon, sfhandle) != 0)
+		{
+			fprintf(stderr, _("%s: Close failed for %s\n"), ProgramName, source);
+			if (!retValue)
+				retValue = 1;
 		}
 	}
-	if (retryCount >= MaxNcopyRetries)
-		retValue = 1;
-	if (optVerbose)
-		printf("\n");
-	if (ncp_close_file(dcon, dfhandle) != 0)
-	{
-		fprintf(stderr, _("%s: Close failed for %s\n"), ProgramName, destination);
-		retValue = 1;
+	if ((optMAC || optMACinRsrc) && !retValue) {
+		NWCONN_HANDLE sMAC = NULL;
+		NWCONN_HANDLE dMAC = NULL;
+		int sFD = -1;
+		int dFD = -1;
+		const char* sourceRsrc = source;
+		const char* destinationRsrc = destination;
+		char sourceB[MAXPATHLEN+1];
+		char destinationB[MAXPATHLEN+1];
+
+		if (scon && optMAC) {
+			struct nw_info_struct2 srcMAC;
+		
+			err = ncp_ns_obtain_entry_info(scon, NW_NS_DOS, SA_ALL,
+				1, sentry.volume, sentry.dirEnt, NULL, 0,
+				NW_NS_MAC, RIM_DIRECTORY,
+				&srcMAC, sizeof(srcMAC));
+			if (err) {
+				/* it is not fatal... it is user problem... */
+				dfprintf(stderr, _("MAC namespace is not supported on source %s: %s\n"), source, strnwerror(err));
+				/* look for .rsrc */
+			} else {
+				err = ncp_ns_open_create_entry(scon, NW_NS_MAC, SA_ALL,
+					1, srcMAC.Directory.volNumber, srcMAC.Directory.dirEntNum, NULL, 0,
+					1, OC_MODE_OPEN, 0, AR_READ_ONLY,
+					0, 
+					NULL, 0, NULL, NULL, sfhandle);
+				if (err) {
+					/* NWE_SERVER_FAILURE is returned on NSS volume if there does not exist resource fork yet... */
+					/* ignore silently */
+					if (err != NWE_SERVER_FAILURE) {
+						fprintf(stderr, _("Unable to open MAC resource fork on source %s: %s\n"), source, strnwerror(err));
+						retValue = 1;
+					}
+					/* no MAC... */
+					goto nomac;
+				}
+				sMAC = scon;
+			}
+		}
+		if (!sMAC) {
+			const char* p;
+			char* d;
+			
+			if (!optMACinRsrc)
+				goto nomac;
+			
+			p = strrchr(source, '/');
+			if (p)
+				p = p + 1;
+			else
+				p = source;
+			memcpy(sourceB, source, p - source);
+			d = sourceB + (p - source);
+			d = stpcpy(d, RSRCSUBDIR);
+			strcpy(d, p);
+			sFD = open(sourceB, O_RDONLY);
+			if (sFD == -1)
+				goto nomac;
+			sourceRsrc = sourceB;
+		}
+		if (dcon && optMAC) {
+			struct nw_info_struct2 dstMAC;
+
+			err = ncp_ns_obtain_entry_info(dcon, NW_NS_DOS, SA_ALL,
+				1, dentry.volume, dentry.dirEnt, NULL, 0,
+				NW_NS_MAC, RIM_DIRECTORY,
+				&dstMAC, sizeof(dstMAC));
+			if (err) {
+				/* possible information loss... maybe fallback to create .rsrc/name */
+				fprintf(stderr, _("MAC namespace is not supported on destination %s: %s\n"), destination, strnwerror(err));
+				retValue = 1;
+			} else {
+				err = ncp_ns_open_create_entry(dcon, NW_NS_MAC, SA_ALL,
+					1, dstMAC.Directory.volNumber, dstMAC.Directory.dirEntNum, NULL, 0,
+					1, OC_MODE_OPEN, 0, AR_WRITE_ONLY,
+					0,
+					NULL, 0, NULL, NULL, dfhandle);
+				if (err) {
+					/* it is fatal... */
+					fprintf(stderr, _("Unable to create MAC resource fork on destination %s: %s\n"), destination, strnwerror(err));
+					retValue = 1;
+					goto nomac;
+				}
+				dMAC = dcon;
+			}
+		}
+		if (!dMAC) {
+			const char* p;
+			char* d;
+			
+			if (!optMACinRsrc) {
+				/* try to create .rsrc/destination */
+				fprintf(stderr, _("Unable to copy MAC resource fork of %s because of %s does not support resource forks\n"),
+					source,
+					destination);
+					retValue = 1;
+				goto nomac;
+			}
+			
+			p = strrchr(destination, '/');
+			if (p)
+				p = p + 1;
+			else
+				p = destination;
+			memcpy(destinationB, destination, p - destination);
+			d = destinationB + (p - destination);
+			d = stpcpy(d, RSRCSUBDIR);
+			strcpy(d, p);
+			dFD = open(destinationB, O_CREAT | O_TRUNC | O_WRONLY, sourceMode);
+			if (dFD == -1) {
+				if (errno == ENOENT) {
+					*(d-1) = 0;
+					mkdir(destinationB, sourceMode | ((sourceMode & 0444) >> 2) | ((sourceMode & 0222) >> 1));
+					*(d-1) = '/';
+					dFD = open(destinationB, O_CREAT | O_TRUNC | O_WRONLY, sourceMode);
+				}
+				if (dFD == -1) {
+					int err2 = errno;
+					
+					fprintf(stderr, _("Unable to copy MAC resource fork: %s: %s\n"),
+						destinationB,
+						strerror(err2));
+					retValue = 1;
+					goto nomac;
+				}
+			}
+			destinationRsrc = destinationB;
+		}
+		retValue = copyNWNW(_("NetWare copy (resource fork)"), 
+			    sFD, sMAC, sfhandle, sourceRsrc,
+			    dFD, dMAC, dfhandle, destinationRsrc);
+nomac:;
+		if (dMAC)
+			ncp_close_file(dMAC, dfhandle);
+		if (dFD != -1)
+			close(dFD);
+		if (sMAC)
+			ncp_close_file(sMAC, sfhandle);
+		if (sFD != -1)
+			close(sFD);
 	}
-	CurrentFile = NULL;
-	if (ncp_close_file(scon, sfhandle) != 0)
-	{
-		fprintf(stderr, _("%s: Close failed for %s\n"), ProgramName, source);
-		if (!retValue)
-			retValue = 1;
-	}
-	if ((optTrustees) && (retValue == 0)) {
+	if ((optTrustees) && (retValue == 0) && scon) {
 		u_int32_t iter = 0;
 		while (1) {
 			TRUSTEE_INFO ts[128];
@@ -534,13 +695,19 @@ netwareCopyFile(int fdIn, int fdOut,
 			unsigned int sptr;
 			unsigned int dptr;
 				
-			if (ncp_ns_trustee_scan(scon, NW_NS_DOS, 0x8006,
+			if (ncp_ns_trustee_scan(scon, NW_NS_DOS, SA_ALL,
 					1, sentry.volume, sentry.dirEnt, 
 					NULL, 0,
 					&iter, ts, &tstcount))
 				break;
 			if (tstcount == 0)
 				break;
+			if (!dcon) {
+				fprintf(stderr, _("Cannot set trustees on %s because of %s\n"),
+					destination,
+					_("not NetWare filesystem"));
+				break;
+			}
 			dptr = 0;
 			for (sptr = 0; sptr < tstcount; sptr++) {
 				if (xlatid(scon, ts[sptr].objectID, dcon, &ts[dptr].objectID)) {
@@ -549,7 +716,7 @@ netwareCopyFile(int fdIn, int fdOut,
 				}
 			}
 			if (dptr) {
-				err = ncp_ns_trustee_add(dcon, NW_NS_DOS, 0x8006,
+				err = ncp_ns_trustee_add(dcon, NW_NS_DOS, SA_ALL,
 					1, dentry.volume, dentry.dirEnt, 
 					NULL, 0,
 					ts, dptr, 0xFFFF);
@@ -561,64 +728,60 @@ netwareCopyFile(int fdIn, int fdOut,
 			}
 		}
 	}
-	if ((optOwner || optFlags || optTimes) && (retValue == 0)) {
-		struct ncp_dos_info nwmd;
-		u_int32_t mask = 0;
-		
-		memset(&nwmd, 0, sizeof(nwmd));
-		if (optOwner) {
-			if (xlatid(scon, sfi.Creation.ID, dcon, &nwmd.Creation.ID))
-				mask |= DM_CREATOR_ID;
-			if (xlatid(scon, sfi.Modify.ID, dcon, &nwmd.Modify.ID))
-				mask |= DM_MODIFIER_ID;
-			if (xlatid(scon, sfi.Archive.ID, dcon, &nwmd.Archive.ID))
-				mask |= DM_ARCHIVER_ID;
-		}
-		if (optFlags) {
-			nwmd.Attributes = sfi.Attributes.Attributes;
-			nwmd.Rights.Grant = sfi.Rights;
-			nwmd.Rights.Revoke = ~sfi.Rights;
-			mask |= DM_ATTRIBUTES | DM_INHERITED_RIGHTS_MASK;
-		}
-		if (optTimes) {
-			nwmd.Creation.Date = sfi.Creation.Date;
-			nwmd.Creation.Time = sfi.Creation.Time;
-			nwmd.Modify.Date = sfi.Modify.Date;
-			nwmd.Modify.Time = sfi.Modify.Time;
-			nwmd.Archive.Date = sfi.Archive.Date;
-			nwmd.Archive.Time = sfi.Archive.Time;
-			nwmd.LastAccess.Date = sfi.LastAccess.Date;
-			mask |= DM_CREATE_DATE | DM_CREATE_TIME |
-			        DM_MODIFY_DATE | DM_MODIFY_TIME |
-				DM_ARCHIVE_DATE | DM_ARCHIVE_TIME |
-				DM_LAST_ACCESS_DATE;
-		}
-		if (mask) {
-			err = ncp_ns_modify_entry_dos_info(
-				dcon, NW_NS_DOS, 0x8006, 
-				1, dentry.volume, dentry.dirEnt, NULL, 0,
-				mask, &nwmd);
-			if (err) {
-				fprintf(stderr, _("Cannot set file attributes on %s because of %s\n"),
-					destination,
-					strnwerror(err));
+	if ((optOwner || optFlags || optTimes) && (retValue == 0) && scon) {
+		if (!dcon) {
+			fprintf(stderr, _("Cannot set file attributes on %s because of %s\n"),
+				destination,
+				_("not NetWare filesystem"));
+		} else {
+			struct ncp_dos_info nwmd;
+			u_int32_t mask = 0;
+			memset(&nwmd, 0, sizeof(nwmd));
+			if (optOwner) {
+				if (xlatid(scon, sfi.Creation.ID, dcon, &nwmd.Creation.ID))
+					mask |= DM_CREATOR_ID;
+				if (xlatid(scon, sfi.Modify.ID, dcon, &nwmd.Modify.ID))
+					mask |= DM_MODIFIER_ID;
+				if (xlatid(scon, sfi.Archive.ID, dcon, &nwmd.Archive.ID))
+					mask |= DM_ARCHIVER_ID;
+			}
+			if (optFlags) {
+				nwmd.Attributes = sfi.Attributes.Attributes;
+				nwmd.Rights.Grant = sfi.Rights;
+				nwmd.Rights.Revoke = ~sfi.Rights;
+				mask |= DM_ATTRIBUTES | DM_INHERITED_RIGHTS_MASK;
+			}
+			if (optTimes) {
+				nwmd.Creation.Date = sfi.Creation.Date;
+				nwmd.Creation.Time = sfi.Creation.Time;
+				nwmd.Modify.Date = sfi.Modify.Date;
+				nwmd.Modify.Time = sfi.Modify.Time;
+				nwmd.Archive.Date = sfi.Archive.Date;
+				nwmd.Archive.Time = sfi.Archive.Time;
+				nwmd.LastAccess.Date = sfi.LastAccess.Date;
+				mask |= DM_CREATE_DATE | DM_CREATE_TIME |
+				        DM_MODIFY_DATE | DM_MODIFY_TIME |
+					DM_ARCHIVE_DATE | DM_ARCHIVE_TIME |
+					DM_LAST_ACCESS_DATE;
+			}
+			if (mask) {
+				err = ncp_ns_modify_entry_dos_info(
+					dcon, NW_NS_DOS, SA_ALL, 
+					1, dentry.volume, dentry.dirEnt, NULL, 0,
+					mask, &nwmd);
+				if (err) {
+					fprintf(stderr, _("Cannot set file attributes on %s because of %s\n"),
+						destination,
+						strnwerror(err));
+				}
 			}
 		}
 	}
 	/* Clear signal handling globals */
 	CurrentConn = NULL;
-	goto cdcon;
-csfi:;
-	if (ncp_close_file(scon, sfhandle) != 0)
-	{
-		fprintf(stderr, _("%s: Close failed for %s\n"), ProgramName, source);
-		if (!retValue)
-			retValue = 1;
-	}
-cdcon:;
-	ncp_close(dcon);
-cscon:;
-	if (!ncopy)
+	if (dcon)
+		ncp_close(dcon);
+	if (scon && !ncopy)
 		ncp_close(scon);
 	return retValue;
 }
@@ -632,7 +795,6 @@ static int
 copyFiles(const char *source, const char *destination)
 {
 	int oldUMask;
-	char fileBuffer[24000];
 	int retVal;
 	int fdIn, fdOut;
 	struct stat statBuf;
@@ -673,10 +835,7 @@ copyFiles(const char *source, const char *destination)
 		return 1;
 	}
 
-	retVal = netwareCopyFile(fdIn, fdOut, source, destination);
-	if (retVal >= 2) /* did not start... */
-		retVal = normalFileCopy(fdIn, fdOut, source, destination, fileBuffer,
-					sizeof(fileBuffer));
+	retVal = netwareCopyFile(fdIn, fdOut, source, destination, statBuf.st_mode);
 	close(fdOut);
 	umask(oldUMask);
 	close(fdIn);
@@ -866,7 +1025,11 @@ int
 main(int argc, char *const argv[])
 {
 	int returnCode;
-	ProgramName = argv[0];
+	ProgramName = strrchr(argv[0], '/');
+	if (ProgramName)
+		ProgramName++;
+	else
+		ProgramName = argv[0];
 
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
