@@ -1,11 +1,28 @@
 /*
- *  nwlsobj.c
- *
- *  List bindery objects
- *
- *  Copyright (C) 1996 by Volker Lendecke
- *
- */
+    nwbols.c - List bindery objects
+    Copyright (C) 1996 by Volker Lendecke
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+    Revision history:
+
+	0.00  1996			Volker Lendecke
+		Initial revision.
+
+*/
 
 #include "config.h"
 
@@ -44,6 +61,7 @@ help(void)
 	       "-o object      Object pattern\n"
 	       "-v             Verbose listing\n"
 	       "-a             Alternative output format\n"
+	       "-d             Show object type in decimal\n"
 	       "\n"));
 }
 
@@ -61,6 +79,7 @@ main(int argc, char **argv)
 	int opt;
 	int verbose = 0;
 	int fmt = 0;
+	int decl = 0;
 	u_int16_t type = 0xffff;
 	int useConn = 0;
 
@@ -74,7 +93,7 @@ main(int argc, char **argv)
 	{
 		useConn = 1;
 	}
-	while ((opt = getopt(argc, argv, "h?vt:o:a")) != EOF)
+	while ((opt = getopt(argc, argv, "h?vt:o:ad")) != EOF)
 	{
 		switch (opt)
 		{
@@ -94,6 +113,9 @@ main(int argc, char **argv)
 		case 'a':
 			fmt = 1;
 			break;
+		case 'd':
+			decl = 1;
+			break;
 		default:
 			usage();
 			exit(1);
@@ -132,31 +154,42 @@ main(int argc, char **argv)
 		if (verbose != 0)
 		{
 			if (fmt) {
-				printf("%08X %04X %-48s %d %02X %d\n",
-				       (u_int32_t)o.object_id,
-				       o.object_type,
-				       o.object_name,
-				       o.object_flags,
-				       o.object_security,
-				       o.object_has_prop);
+				printf("%08X ", o.object_id);
+				if (decl) {
+					printf("%05u ", o.object_type);
+				} else {
+					printf("%04X ", o.object_type);
+				}
+				printf("%-48s %u %02X %u\n",
+				       o.object_name, o.object_flags,
+				       o.object_security, o.object_has_prop);
 			} else {
-				printf("%s %08X %04X %d %02X %d\n",
-				       o.object_name, (unsigned int) o.object_id,
-				       (unsigned int) o.object_type,
-				       o.object_flags, o.object_security,
-				       o.object_has_prop);
+				printf("%s %08X ", o.object_name, o.object_id);
+				if (decl) {
+					printf("%05u ", o.object_type);
+				} else {
+					printf("%04X ", o.object_type);
+				}
+				printf("%u %02X %u\n", o.object_flags,
+				       o.object_security, o.object_has_prop);
 			}
 		} else
 		{
 			if (fmt) {
-				printf("%08X %04X %s\n",
-				       (u_int32_t)o.object_id,
-				       o.object_type,
-				       o.object_name);
+				printf("%08X ", o.object_id);
+				if (decl) {
+					printf("%05u ", o.object_type);
+				} else {
+					printf("%04X ", o.object_type);
+				}
+				printf("%s\n", o.object_name);
 			} else {
-				printf("%s %08X %04X\n",
-				      o.object_name, (unsigned int) o.object_id,
-				      (unsigned int) o.object_type);
+				printf("%s %08X ", o.object_name, o.object_id);
+				if (decl) {
+					printf("%05u\n", o.object_type);
+				} else {
+					printf("%04X\n", o.object_type);
+				}
 			}
 		}
 	}
