@@ -1,10 +1,30 @@
 /*
- *  nwbpadd.c
- *
- *  Set the contents of a SET property of a bindery object on a NetWare server
- *
- *  Copyright (C) 1996 by Volker Lendecke
- *
+    nwbpadd.c - Set the contents of a SET property of a bindery object on a NetWare server
+    Copyright (C) 1996 by Volker Lendecke
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+    Revision history:
+
+	0.00  1996			Volker Lendecke
+		Initial revision.
+
+	1.00  2000, June 1		Bruce Richardson <brichardson@lineone.net>
+		Swapped object ID to match with other tools.
+
  */
 
 #include "config.h"
@@ -63,6 +83,7 @@ main(int argc, char *argv[])
 	long err;
 	int result = 1;
 	int opt;
+	int dontohl = 0;
 
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
@@ -75,7 +96,7 @@ main(int argc, char *argv[])
 		com_err(argv[0], err, _("when initializing"));
 		goto finished;
 	}
-	while ((opt = getopt(argc, argv, "h?o:t:p:v:")) != EOF)
+	while ((opt = getopt(argc, argv, "h?o:t:p:v:O")) != EOF)
 	{
 		switch (opt)
 		{
@@ -99,6 +120,9 @@ main(int argc, char *argv[])
 		case 'v':
 			value = optarg;
 			break;
+		case 'O':
+			dontohl = 1;
+			break;
 		case 'h':
 		case '?':
 			help();
@@ -146,7 +170,8 @@ main(int argc, char *argv[])
 	{
 		/* Property is of type SET */
 		struct ncp_bindery_object o;
-
+		NWObjectID id;
+		
 		if (optind != argc)
 		{
 			fprintf(stderr, _("%s: For the SET property %s, you must"
@@ -154,8 +179,11 @@ main(int argc, char *argv[])
 				progname, property_name);
 			goto finished;
 		}
+		id = strtoul(value, NULL, 16);
+		if (dontohl)
+			id = ntohl(id);
 		if (ncp_get_bindery_object_name(conn,
-						ntohl(strtol(value, NULL, 16)),
+						id,
 						&o) != 0)
 		{
 			fprintf(stderr, _("%s: %s is not a valid object id\n"),
