@@ -1,10 +1,33 @@
 /*
- *  nwdir.c - list contents of directory
- *
- *  Copyright (c) 1998 Milan Vandrovec
- *
- *  Updated for new ncp_ns_* interface by Petr Vandrovec, 1999.
- *
+    nwdir.c - list contents of directory
+    Copyright (c) 1998 Milan Vandrovec
+  
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+    Revision history:
+
+	0.00  1998			Milan Vandrovec
+		Initial revision.
+
+	0.01  1999			Petr Vandrovec <vandrove@vc.cvut.cz>
+		Updated for new ncp_ns_* interface.
+
+	1.00  1999, November 20		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Added license.
+  
  */
  
 #include "config.h"
@@ -581,11 +604,7 @@ int main(int argc, char* argv[]) {
   }
   err = NWParsePath(dpath, NULL, &conn, volname, dirpath);
   if (err) {
-    if (err == 0x880F) {
-      fprintf(stderr, _("You are not attached to specified server\n"));
-    } else {
-      fprintf(stderr, _("Unable to parse path: 0x%04X\n"), err);
-    }
+    fprintf(stderr, _("Invalid path: %s\n"), strnwerror(err));
     return 123;
   }
   if (!conn) {
