@@ -1,10 +1,29 @@
 /*
- *  nwfsctrl.c
- *
- *  Down file server, open bindery, close bindery
- *
- *  Copyright (C) 1998 by Petr Vandrovec <vandrove@vc.cvut.cz>
- *
+    nwfsctrl.c - Down file server, open bindery, close bindery
+    Copyright (C) 1998, 1999  Petr Vandrovec
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+    Revision history:
+
+	0.00  1998			Petr Vandrovec <vandrove@vc.cvut.cz>
+		Initial revision.
+
+	1.00  1999, November 20		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Added license.
  */
 
 #include "config.h"
@@ -268,7 +287,7 @@ static int cmd_clear_station(char* id) {
 	int err = 0;
 	
 	while (*id) {
-		unsigned long int connNum;
+		NWCONN_NUM connNum;
 		char* ptr;
 
 		connNum = strtoul(id, &ptr, 0);
@@ -277,13 +296,13 @@ static int cmd_clear_station(char* id) {
 			
 			nwerr = NWClearConnectionNumber(conn, connNum);
 			if (nwerr) {
-				fprintf(stderr, _("%s: Unable to clear station %d, error 0x%04X (%u)\n"),
+				fprintf(stderr, _("%s: Unable to clear station %u, error 0x%04X (%u)\n"),
 					progname, connNum, nwerr, nwerr);
 				err = 100;
 			}
 		} else {
 			ptr = skipword(id);
-			fprintf(stderr, _("%s: `%.*s' is not number, skipping\n"), progname, ptr-id, id);
+			fprintf(stderr, _("%s: `%.*s' is not a number, skipping\n"), progname, ptr-id, id);
 			if (!err) err = 121;
 		}
 		id = skipspaces(ptr);
