@@ -1,12 +1,33 @@
 /*
- *  nwrevoke.c
- *
- *  Remove trustee rights from file or directory
- *
- *  Copyright (C) 1996 by Volker Lendecke
- *
- *  Netware 3.11 and above support by Petr Vandrovec, 1999.
- *
+    nwrevoke.c - Remove trustee rights from file or directory
+    Copyright (C) 1996 by Volker Lendecke
+  
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+    Revision history:
+
+	0.00  1996			Volker Lendecke
+		Initial revision.
+
+	0.01  1999			Petr Vandrovec <vandrove@vc.cvut.cz>
+		Support for Netware 3.11 and above.
+
+	1.00  1999, November 20		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Added license.
+  
  */
 
 #include "config.h"
@@ -140,7 +161,7 @@ main(int argc, char *argv[])
 	}
 	enclen = ncp_path_to_NW_format(path, encpath, sizeof(encpath));
 	if (enclen < 0) {
-		fprintf(stderr, _("%s: Invalid path: %s\n"), strerror(-enclen));
+		fprintf(stderr, _("%s: Invalid path: %s\n"), progname, strerror(-enclen));
 		goto finished;
 	}
 	tstinfo.objectID = o.object_id;
