@@ -1,10 +1,30 @@
 /*
- *  nwrights.c
- *
- *  Show effective rights for dir or file.
- *
- *  Copyright (C) 1996 by Volker Lendecke
- *
+    nwrights.c - Show effective rights for dir or file.
+    Copyright (C) 1996 by Volker Lendecke
+  
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+    Revision history:
+
+	0.00  1996			Volker Lendecke
+		Initial revision.
+		
+	1.00  2000, May 18		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Modified to use ncp_perms_to_str.
+
  */
 
 #include "config.h"
@@ -42,6 +62,7 @@ main(int argc, char *argv[])
 	struct NWCCRootEntry root;
 	struct ncp_conn *conn = NULL;
 	const char *path = ".";
+	char permbuf[11];
 	long err;
 	int result = 1;
 	int opt;
@@ -95,16 +116,9 @@ main(int argc, char *argv[])
 		com_err(argv[0], err, _("when finding rights"));
 		goto finished;
 	}
-	printf(_("Your effective rights for %s are: [%c%c%c%c%c%c%c%c]\n"),
+	printf(_("Your effective rights for %s are: %s\n"),
 	       path,
-	       ((rights & NCP_PERM_SUPER) != 0) ? 'S' : ' ',
-	       ((rights & NCP_PERM_READ) != 0) ? 'R' : ' ',
-	       ((rights & NCP_PERM_WRITE) != 0) ? 'W' : ' ',
-	       ((rights & NCP_PERM_CREATE) != 0) ? 'C' : ' ',
-	       ((rights & NCP_PERM_DELETE) != 0) ? 'E' : ' ',
-	       ((rights & NCP_PERM_MODIFY) != 0) ? 'M' : ' ',
-	       ((rights & NCP_PERM_SEARCH) != 0) ? 'F' : ' ',
-	       ((rights & NCP_PERM_OWNER) != 0) ? 'A' : ' ');
+	       ncp_perms_to_str(permbuf, rights));
 
 	if ((rights & NCP_PERM_SUPER) != 0)
 	{
