@@ -1,19 +1,41 @@
 /*
- *  slist.c
- *
- *  List all file server that are known in the IPX network.
- *
- *  Copyright (C) 1995 by Volker Lendecke
- *
+    slist.c - List all file server that are known in the IPX network.
+    Copyright (C) 1995 by Volker Lendecke
+  
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+    Revision history:
+
+	0.00  1995			Volker Lendecke
+		Initial revision.
  */
 
-#include "ncplib.h"
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <ctype.h>
 
+#include <ncp/ncplib.h>
+
+#include <libintl.h>
+#define _(X) gettext(X)
+
 int
 main(int argc, char *argv[])
 {
@@ -25,9 +47,13 @@ main(int argc, char *argv[])
 	char *p;
 	long err;
 
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+	
 	if (argc > 2)
 	{
-		printf("usage: %s [pattern]\n", argv[0]);
+		printf(_("usage: %s [pattern]\n"), argv[0]);
 		exit(1);
 	}
 	if (argc == 2)
@@ -41,7 +67,7 @@ main(int argc, char *argv[])
 
 	if ((conn = ncp_open(NULL, &err)) == NULL)
 	{
-		com_err(argv[0], err, "in ncp_open");
+		com_err(argv[0], err, _("in ncp_open"));
 		exit(1);
 	}
 	if (isatty(1))
@@ -49,9 +75,9 @@ main(int argc, char *argv[])
 		printf("\n%-52s%-10s%-12s\n"
 		       "-----------------------------------------------"
 		       "---------------------------\n",
-		       "Known NetWare File Servers",
-		       "Network",
-		       "Node Address");
+		       _("Known NetWare File Servers"),
+		       _("Network"),
+		       _("Node Address"));
 	}
 	obj.object_id = 0xffffffff;
 
@@ -80,7 +106,7 @@ main(int argc, char *argv[])
 
 	if ((found == 0) && (isatty(1)))
 	{
-		printf("No servers found\n");
+		printf(_("No servers found\n"));
 	}
 	ncp_close(conn);
 	return 0;
